# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime


parser_class = "alere_reader"


def test_decode_result(self):
    _result = """
    <?xml version="1.0" encoding="utf-8"?>
    <OBS.R01>
        <HDR>
            <HDR.control_id V="3" />
            <HDR.version_id V="POCT1" />
            <HDR.creation_dttm V="2017-02-21T15:16:34-08:00" />
        </HDR>
        <SVC>
            <SVC.role_cd V="OBS" />
            <SVC.observation_dttm V="2017-02-21T15:04:21-08:00" />
            <SVC.status_cd V="NRM" />
            <SVC.reason_cd V="NEW" />
            <PT>
                <PT.patient_id V="35609175" />
                <PT.name V="John Q. Doe" />
                <PT.birth_date V="1999-08-21" />
                <OBS>
                    <OBS.observation_id V="Flu B" SN="ALERE.SCR" />
                    <OBS.collection_method_cd V="swab" SN="ALERE.SCR" />
                    <OBS.qualitative_value V="positive" />
                    <OBS.method_cd V="M" />
                    <OBS.status_cd V="A" />
                </OBS>
                <OBS>
                    <OBS.observation_id V="Flu A" SN="ALERE.SCR" />
                    <OBS.collection_method_cd V="swab" SN="ALERE.SCR" />
                    <OBS.qualitative_value V="negative" />
                    <OBS.method_cd V="M" />
                    <OBS.status_cd V="A" />
                </OBS>
            </PT>
            <OPR>
                <OPR.operator_id V="291054" />
            </OPR>
            <RGT>
                <RGT.name V="BinaxNOW Flu A+B" />
                <RGT.lot_number V="002209902123456" />
                <RGT.expiration_date V="2099-02-28" />
            </RGT>
        </SVC>
    </OBS.R01>
    """
    worklist = self.decode(_result)
    date = datetime.datetime(2017, 2, 21, 15, 4, 21)
    expiration_date = datetime.datetime(2099, 2, 28, 0, 0)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "Flu B",
                        "date": date,
                        "lots": [
                            {
                                "expiration": expiration_date,
                                "lot": "002209902123456",
                                "name": "BinaxNOW Flu A+B",
                                "type": "reagent",
                            }
                        ],
                        "operator": "291054",
                        "value": "positive",
                    },
                    {
                        "code": "Flu A",
                        "date": date,
                        "lots": [
                            {
                                "expiration": expiration_date,
                                "lot": "002209902123456",
                                "name": "BinaxNOW Flu A+B",
                                "type": "reagent",
                            }
                        ],
                        "operator": "291054",
                        "value": "negative",
                    },
                ],
                "sample_id": "35609175",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    _control = """
    <?xml version="1.0" encoding="utf-8"?>
    <OBS.R02>
        <HDR>
            <HDR.control_id V="4" />
            <HDR.version_id V="POCT1" />
            <HDR.creation_dttm V="2017-02-21T19:19:34-08:00" />
        </HDR>
        <SVC>
            <SVC.role_cd V="LQC" />
            <SVC.observation_dttm V="2017-02-21T14:52:37-08:00" />
            <SVC.status_cd V="NRM" />
            <SVC.reason_cd V="NEW" />
            <CTC>
                <CTC.name V="Control swab" />
                <CTC.lot_number V="149240" />
                <CTC.expiration_date V="2050-05-31" />
                <CTC.level_cd V="pos" />
                <OBS>
                    <OBS.observation_id V="Flu A" SN="ALERE.SCR" />
                    <OBS.qualitative_value V="pass" />
                    <OBS.method_cd V="M" />
                    <OBS.status_cd V="A" />
                </OBS>
                <OBS>
                    <OBS.observation_id V="Flu B" SN="ALERE.SCR" />
                    <OBS.qualitative_value V="fail" />
                    <OBS.method_cd V="M" />
                    <OBS.status_cd V="A" />
                </OBS>
            </CTC>
            <OPR>
                <OPR.operator_id V="291054" />
            </OPR>
            <RGT>
                <RGT.name V="BinaxNOW Flu A+B" />
                <RGT.lot_number V="002209902123456" />
                <RGT.expiration_date V="2099-02-28" />
            </RGT>
        </SVC>
    </OBS.R02>
    """
    worklist = self.decode(_control)
    date = datetime.datetime(2017, 2, 21, 14, 52, 37)
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "149240",
                "control_level": "pos",
                "control_lot": "149240",
                "control_name": "Control swab",
                "results": [
                    {"code": "Flu A", "date": date, "value": "pass"},
                    {"code": "Flu B", "date": date, "value": "fail"},
                ],
                "type": "control",
            }
        ],
    }
    assert worklist == expected
