# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "c111"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1
    O|1|123456||^^^A^1:10\\^^^B|R||||||A||||||||||||||O
    O|2|123456||^^^C|R||||||C||||||||||||||O
    L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||c111_DB_B^Roche^0.5.1.0505^PROTO_SIM|||||host|TSREQ^REAL|P|1|20160208142948
    Q|1|^123456||ALL||||||||O
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123456"}],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2016, 2, 8, 15, 15, 15)
    worklist = self.decode(
        """
    H|\\^&|||c111^Roche^c111^0.5.4.0509^1^1005|||||host|RSUPL^BATCH|P|1|20160208152001
    P|1
    O|1||123456^A^001||R||||||N|||||||||||20160208151515|||F
    R|1|^^^111|1480.00|mmol/l||N||F||UnitTest
    R|2|^^^121|1613.33|mmol/l||N||F||UnitTest
    C|1|I|123^Limit hight\\18^Ise Unstab|I
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "rack": "A-001",
                "results": [
                    {
                        "code": "111",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "value": "1480.00",
                        "operator": "UnitTest",
                    },
                    {
                        "code": "121",
                        "comments": ["Limit hight", "Ise Unstab"],
                        "date": date,
                        "flags": ["N", "123", "18"],
                        "value": "1613.33",
                        "operator": "UnitTest",
                    },
                ],
                "sample_id": "123456",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2016, 2, 8, 15, 15, 15)
    worklist = self.decode(
        """
    H|\\^&|||c111^Roche^c111^0.5.4.0509^1^1005|||||host|RSUPL^BATCH|P|1|20160208152001
    P|1
    O|1|PCCC1^18173800^20170831|1391^A^001|^^^221|||||||Q|||||||||||20160208151515|||F
    R|1|^^^212|2826.67|mmol/l||||R||UnitTest
    R|2|^^^221|2946.67|mmol/l||||R||UnitTest
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "18173800-1391",
                "sample_id": "18173800-1391",
                "control_level": "1391",
                "rack": "A-001",
                "results": [
                    {
                        "code": "212",
                        "value": "2826.67",
                        "date": date,
                        "flags": [],
                        "comments": [],
                    },
                    {
                        "code": "221",
                        "value": "2946.67",
                        "date": date,
                        "flags": [],
                        "comments": [],
                    },
                ],
            }
        ],
    }
    assert worklist == expected
