# -*- coding: utf8 -*-
from __future__ import unicode_literals


parser_class = "cellmek"


def test_resu_rejet(self):
    resu_rejet = """
        H|\\^&
        P|1||||||||||||||||||||||||
        O|1|SAMPLE001|||||||||||||||||||||||X
        C|1|I|Reason for rejection.|G
        L|1|N
        """
    result = self.decode(resu_rejet)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "REJET",
                        "comments": ["Reason for rejection."],
                        "value": "X",
                    }
                ],
                "sample_id": "SAMPLE001",
                "type": "patient",
            }
        ],
    }
    assert result == expected


def test_order(self):
    message = self.order()
    expected = """
        H|\\^&|||MCA||||||||LIS2-A2|20130715141912
        P|1||036666666||NYME^ANO||20110513|F|||||ATTENDING^LAGRANGE^Joseph-Louis||||||||||||LOC
        O|1|123456||^^^A\\^^^B|R||20130513114242||||A||||Serum||||||||||O
        O|2|123456||^^^C|R||20130513114242||||C||||Serum||||||||||O
        L|1|N
        """
    assert message == expected


def test_query(self):
    query = """
        H|\\^&|||CellMekSPS^100||||||||LIS2-A2|20121004190802
        Q|1|^036666666||||||||||O
        L|1|N
        """
    result = self.decode(query)
    expected = {
        "context": "query",
        "samples": [{"sample_id": "036666666", "type": "patient"}],
    }
    assert result == expected


def test_query_order_with_void_order(self):
    query = """
        H|\\^&|||CellMekSPS^100||||||||LIS2-A2|20121004190802
        Q|1|^1000||||||||||O
        Q|2|^036666666||||||||||O
        Q|3|^1001||||||||||O
        L|1|N
        """

    worklist = self.encode(self.order_worklist, context={"query": self.parse(query)})
    expected = """
        H|\\^&|||MCA||||||||LIS2-A2|20130715141912
        P|1||036666666||NYME^ANO||20110513|F|||||ATTENDING^LAGRANGE^Joseph-Louis||||||||||||LOC
        O|1|123456||^^^A\\^^^B|R||20130513114242||||A||||Serum||||||||||O
        O|2|123456||^^^C|R||20130513114242||||C||||Serum||||||||||O
        P|2
        O|1|1000|||||||||||||||||||||||Y
        P|3
        O|1|1001|||||||||||||||||||||||Y
        L|1|N
        """
    assert worklist == expected
