# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "ctlink"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||P|1|20130715141912
    P|1|036666666|||NYME^ANO|NJF|20110513121500|F|||||||||||||||||LOC
    O|1|123456||^^^A\\^^^B|A||20130513114242||||A||||Serum|PRESCRIBER
    O|2|123456||^^^C|A||20130513114242||||C||||Serum|PRESCRIBER
    L|1"""
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||||||CENTRALINK 14.0.4|||P|1|20130620113408
    P|1|08000249419|08000249419||ANO^NYME||19570511|F||||||Human^Human
    O|1|2117126029|||R|20130620084534|20130620063000||||||||WB
    R|1|^^^904^^^^^N|2.5|||||||Operateur^OPE|20130620114042||Advia2120_3
    C|1|L|Portoir_: &XA&10050613&XA&I3&XA&Autre|G
    R|1|^^^905^^^^^N|3.5||||||||20130620114042||Advia2120_3
    C|1|L|Résultat contrôlé&XA&Portoir_: &XA&10050613&XA&I3|G
    C|1|L|HACK&XA&Portoir_: |G
    R|1|^^^VB12^x2^2^^DOSE^N|2613|||*||||^batch|20161129120918|20161129121151|CXPT
    C|1|I|Diluted|I
    C|2|I|Repeat|I
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "2117126029",
                "rack": "10050613-I3",
                "specimen": "WB",
                "results": [
                    {
                        "code": "904",
                        "value": "2.5",
                        "date": datetime.datetime(2013, 6, 20, 11, 40, 42),
                        "instrument": "Advia2120_3",
                        "flags": [],
                        "comments": ["Autre"],
                        "operator": "OPE",
                        "dilution": "",
                    },
                    {
                        "code": "905",
                        "value": "3.5",
                        "date": datetime.datetime(2013, 6, 20, 11, 40, 42),
                        "instrument": "Advia2120_3",
                        "flags": [],
                        "comments": ["Résultat contrôlé", "HACK"],
                        "dilution": "",
                    },
                    {
                        "code": "VB12DOSE",
                        "value": "2613",
                        "date": datetime.datetime(2016, 11, 29, 12, 9, 18),
                        "instrument": "CXPT",
                        "flags": ["Diluted", "Repeat"],
                        "comments": [],
                        "dilution": "2",
                        "operator": "batch",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_hemato(self):
    self.configure(analyseurs_hemato="Advia2120_3;Advia2120_2")
    worklist = self.decode(
        """
    H|\\^&|||||||CENTRALINK 14.0.4|||P|1|20130620113408
    P|1|08000249419|08000249419||ANO^NYME||19570511|F||||||Human^Human
    O|1|2117126029|^176^01||R|20130620084534|20130620063000||||||||WB
    R|1|^^^904^^^^^N|2.5||||||||20130620114042||Advia2120_3
    C|1|I|NT|I
    C|2|I|HE|I
    C|3|I|*|I
    R|1|^^^905^^^^^N|42||||||||20130620114042||Advia2120_2
    C|1|I|HE|I
    R|1|^^^INR^^^^^N|2.5||||||||20130620114042||Stago
    C|1|I|NT|I
    C|2|I|*|I
    L|1"""
    )
    date = datetime.datetime(2013, 6, 20, 11, 40, 42)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "2117126029",
                "rack": "176-01",
                "specimen": "WB",
                "results": [
                    {
                        "code": "904",
                        "value": "2.5",
                        "date": date,
                        "flags": ["NT", "HE", "*"],
                        "instrument": "Advia2120_3",
                        "comments": [],
                        "dilution": "",
                    },
                    {
                        "code": "905",
                        "value": "42",
                        "date": date,
                        "instrument": "Advia2120_2",
                        "flags": ["HE"],
                        "comments": [],
                        "dilution": "",
                    },
                    {
                        "code": "INR",
                        "value": "2.5",
                        "date": date,
                        "instrument": "Stago",
                        "flags": ["NT", "*"],
                        "comments": [],
                        "dilution": "",
                    },
                    {"code": "ALARME", "value": "HE;NT"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_hemato_without_flags(self):
    # Une trame hémato sans alarme doit renvoyer une valeur ALARME
    # vide
    self.configure(analyseurs_hemato="Advia2120_3;Advia2120_2")
    worklist = self.decode(
        """
    H|\\^&|||||||CENTRALINK 14.0.4|||P|1|20130620113408
    P|1|08000249419|08000249419||ANO^NYME||19570511|F||||||Human^Human
    O|1|2117126029|^176^01||R|20130620084534|20130620063000||||||||WB
    R|1|^^^904^^^^^N|2.5||||||||20130620114042||Advia2120_3
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "2117126029",
                "rack": "176-01",
                "specimen": "WB",
                "results": [
                    {
                        "code": "904",
                        "value": "2.5",
                        "date": datetime.datetime(2013, 6, 20, 11, 40, 42),
                        "instrument": "Advia2120_3",
                        "flags": [],
                        "comments": [],
                        "dilution": "",
                    },
                    {"code": "ALARME", "value": ""},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||||||CENTRALINK 14.0.5|||P|1|20140212155312
    P|1|||||||U
    O|1|3619745|||||||||Q||||Serum||||||||||F
    R|1|^^^HIVI^^^^INDX^N|8.2620||||||||20140212161607||Centaur_3
    L|1"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "3619745",
                "control_lot": "3619745",
                "specimen": "Serum",
                "results": [
                    {
                        "code": "HIVIINDX",
                        "value": "8.2620",
                        "date": datetime.datetime(2014, 2, 12, 16, 16, 7),
                        "instrument": "Centaur_3",
                        "comments": [],
                        "flags": [],
                        "dilution": "",
                    }
                ],
            }
        ],
    }
    assert worklist == expected
