# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "dynex_lislink_dsx"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||DYNEXRD-LT||||||||1|20130715141912
    P|1||123456||NYME^ANO||20110513|F|||||||||||||||||SER
    O|1|123456||^^^A|S|20130603110342
    P|2||123456||NYME^ANO||20110513|F|||||||||||||||||SER
    O|1|123456||^^^B|S|20130603110342
    L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||DYNEXRD-LT||||||||1|20101208184458
    Q|1|ALL||||||||||O
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"sample_id": "ALL", "type": "patient"}],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2010, 12, 8, 19, 7, 53)
    worklist = self.decode(
        """
    H|\\^&|||DYNEXRD-LT||||||||1|20101208190753
    P|1||Sample Batch 56-1
    O|1|Sample Batch 56-1||^^^StdsCtrls|R|20101208184500
    R|1|^^^StdsCtrls|0.417 U/ml|U/ml||FR||F||||20101208190753|DYNEXRD-LT
    P|2||Sample Batch 56-2
    O|1|Sample Batch 56-2||^^^StdsCtrls|R|20101208184500
    R|1|^^^StdsCtrls|161.537 U/ml|U/ml||FR||F||||20101208190753|DYNEXRD-LT
    R|2|^^^3QUANGR^TB1-Nil|0.015|||RW||F||||20101208190753|1DSA3750
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "StdsCtrls",
                        "date": date,
                        "flags": ["FR"],
                        "instrument": "DYNEXRD-LT",
                        "unit": "U/ml",
                        "value": "0.417",
                    },
                    {
                        "code": "StdsCtrlsFR",
                        "date": date,
                        "flags": ["FR"],
                        "instrument": "DYNEXRD-LT",
                        "unit": "U/ml",
                        "value": "0.417",
                    },
                ],
                "sample_id": "Sample Batch 56-1",
                "type": "patient",
            },
            {
                "results": [
                    {
                        "code": "StdsCtrls",
                        "date": date,
                        "flags": ["FR"],
                        "instrument": "DYNEXRD-LT",
                        "unit": "U/ml",
                        "value": "161.537",
                    },
                    {
                        "code": "StdsCtrlsFR",
                        "date": date,
                        "flags": ["FR"],
                        "instrument": "DYNEXRD-LT",
                        "unit": "U/ml",
                        "value": "161.537",
                    },
                    {
                        "code": "3QUANGRTB1-Nil",
                        "date": date,
                        "flags": ["RW"],
                        "instrument": "1DSA3750",
                        "unit": "",
                        "value": "0.015",
                    },
                    {
                        "code": "3QUANGRTB1-NilRW",
                        "date": date,
                        "flags": ["RW"],
                        "instrument": "1DSA3750",
                        "unit": "",
                        "value": "0.015",
                    },
                ],
                "sample_id": "Sample Batch 56-2",
                "type": "patient",
            },
        ],
    }
    assert worklist == expected
