# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "liaison"


def test_query1(self):
    result = self.decode(
        """
    H|\\^&|||Liaison|||||LaborEDV|||1|19941115202738
    Q|1|Sample01\\Sample02\\Barcode0815\\12345||ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [
            {"sample_id": "Sample01", "type": "patient"},
            {"sample_id": "Sample02", "type": "patient"},
            {"sample_id": "Barcode0815", "type": "patient"},
            {"sample_id": "12345", "type": "patient"},
        ],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^&|||Liaison|||||LaborEDV|||1|19941115202738
    Q|1|Sample01\\Sample02||ALL||||||||O
    L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&|||MCA|||||Liaison|||1|20130715141912
    L|1|I"""
    assert result == expected


def test_query2(self):
    result = self.decode(
        """
    H|\\^&|||Liaison|||||LaborEDV|||1|19941115202738
    Q|1|Sample01||ALL||||||||O
    Q|2|Sample02||ALL||||||||O
    Q|3|Barcode0815||ALL||||||||O
    Q|4|12345||ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [
            {"sample_id": "Sample01", "type": "patient"},
            {"sample_id": "Sample02", "type": "patient"},
            {"sample_id": "Barcode0815", "type": "patient"},
            {"sample_id": "12345", "type": "patient"},
        ],
    }
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||MCA|||||Liaison|||1|20130715141912
        P|1||036666666||NYME^ANO||20110513|F
        O|1|123456||^^^A^1:10\\^^^B|N|20130603110342|||||||||||||||||||O
        L|1|N"""
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||Liaison|||||LaborEDV|||1|19941115202738
    P|1||PatID01||Meier^Anna||19741001|F|||||MARTINEZ
    O|1|123456789||^^^AFP||19980506000000|||||||||||||||||||F
    R|1|^^^AFP|13.1|IU/ml||H||F||||19980506123145|Liaison
    C|1|I|CALIBRATION_EXPIRED\\BAD_QC|I
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "123456789",
                "type": "patient",
                "results": [
                    {
                        "code": "AFP",
                        "value": "13.1",
                        "flags": ["H", "CALIBRATION_EXPIRED", "BAD_QC"],
                        "date": datetime.datetime(1998, 5, 6, 12, 31, 45),
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^&|||Tango|||||Liaison|||1|20130606171326
    P|1||||^||||||||
    O|1|BORM POS||^^^BorM-2^|||||||||||||||||||||F
    R|1|^^^BorM-2|2.6|Index||N||F||||20130606171301|Tango
    P|2||||^||||||||
    O|1|EBNA IgG neg||^^^EBNA-G^|||||||||||||||||||||F
    R|1|^^^EBNA-G|3.0|U/ml||<||F||||20130606171315|Tango
    P|3||||^||||||||
    O|1|EBNA IgG pos||^^^EBNA-G^|||||||||||||||||||||F
    R|1|^^^EBNA-G|61.3|U/ml||N||F||||20130606171325|Tango
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "BORM POS",
                "type": "control",
                "control_lot": "BORM POS",
                "results": [
                    {
                        "date": datetime.datetime(2013, 6, 6, 17, 13, 1),
                        "value": "2.6",
                        "code": "BorM-2",
                        "flags": ["N"],
                    }
                ],
            },
            {
                "sample_id": "EBNA IgG neg",
                "type": "control",
                "control_lot": "EBNA IgG neg",
                "results": [
                    {
                        "date": datetime.datetime(2013, 6, 6, 17, 13, 15),
                        "value": "3.0",
                        "code": "EBNA-G",
                        "flags": ["<"],
                    }
                ],
            },
            {
                "sample_id": "EBNA IgG pos",
                "type": "control",
                "control_lot": "EBNA IgG pos",
                "results": [
                    {
                        "date": datetime.datetime(2013, 6, 6, 17, 13, 25),
                        "value": "61.3",
                        "code": "EBNA-G",
                        "flags": ["N"],
                    }
                ],
            },
        ],
    }
    assert result == expected


def test_option_ignore_flag_qc(self):
    self.configure(liaison_ignore_flag_qc=True)
    result = self.decode(
        """
    H|\\^&|||Tango|||||Liaison|||1|20130606171326
    P|1||||^||||||||
    O|1|BORM POS||^^^BorM-2^|||||||||||||||||||||F
    R|1|^^^BorM-2|2.6|Index||N||F||||20130606171301|Tango
    P|2||||^||||||||
    O|1|EBNA IgG neg||^^^EBNA-G^|||||||||||||||||||||F
    R|1|^^^EBNA-G|3.0|U/ml||<||F||||20130606171315|Tango
    P|3||||^||||||||
    O|1|EBNA IgG pos||^^^EBNA-G^|||||||||||||||||||||F
    R|1|^^^EBNA-G|61.3|U/ml||N||F||||20130606171325|Tango
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "BORM POS",
                "results": [
                    {
                        "code": "BorM-2",
                        "date": datetime.datetime(2013, 6, 6, 17, 13, 1),
                        "flags": [],
                        "value": "2.6",
                    }
                ],
                "sample_id": "BORM POS",
                "type": "control",
            },
            {
                "control_lot": "EBNA IgG neg",
                "results": [
                    {
                        "code": "EBNA-G",
                        "date": datetime.datetime(2013, 6, 6, 17, 13, 15),
                        "flags": [],
                        "value": "3.0",
                    }
                ],
                "sample_id": "EBNA IgG neg",
                "type": "control",
            },
            {
                "control_lot": "EBNA IgG pos",
                "results": [
                    {
                        "code": "EBNA-G",
                        "date": datetime.datetime(2013, 6, 6, 17, 13, 25),
                        "flags": [],
                        "value": "61.3",
                    }
                ],
                "sample_id": "EBNA IgG pos",
                "type": "control",
            },
        ],
    }
    assert result == expected
