# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "mpl_hl7"


def test_decode_order(self):
    worklist = self.decode(
        """
    MSH|^~\\&|middleware||MPL||20101216095928||OML^O33^OML_O33|MSG0000057367|P|2.5|||AL|NE|
    PID|1|4645615191|0012541258||DUPONT^Sophie|DURAND|20011212|F|||
    NTE|1|P|Infos patient|
    PV1|1||SURG||||DUCH|||||||||||||||||||||||||||||||||||||||
    SPM|1|01||SERUM|
    ORC|NW||0100000667|0100000667|CM||||||||||||
    OBR|1||0100000667|K^POTASSIUM||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|NW||0100000667|0100000667|CM|||||||||||
    OBR|1||0100000667|CL^CHLORINE||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|NW||0100000667|0100000667|CM|||||||||||
    OBR|1||0100000667|CRE^CREATIN||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|NW||0100000667|0100000667|CM|||||||||||
    OBR|1||0100000667|UREA^UREA||||||||||||DUCH^DUCHEMIN|||||||||||^^^20101216095812^^R||||||||||
    ORC|PR||0100000667|0100000667||||||||||||
    OBR|1||0100000667|UREA^UREA|||||||||||||||||||||||^^^20101216095812^^R||||||||||
    NTE|1|P|Obs tube|
    OBX|1|NM|NA||12|mmol/L|||||F|||20110124134900||||||
    """
    )
    expected = {
        "context": "order",
        "sender_id": "middleware",
        "receiver_id": "MPL",
        "samples": [
            {
                "birthdate": datetime.datetime(2001, 12, 12, 0, 0),
                "collection_date": datetime.datetime(2010, 12, 16, 9, 58, 12),
                "first_name": "Sophie",
                "last_name": "DUPONT",
                "location": "MPLSURG",
                "maiden_name": "DURAND",
                "orders": [
                    {"action": "add", "code": "K", "priority": "R"},
                    {"action": "add", "code": "CL", "priority": "R"},
                    {"action": "add", "code": "CRE", "priority": "R"},
                    {"action": "add", "code": "UREA", "priority": "R"},
                ],
                "patient_id": "4645615191",
                "physicians": [
                    {"code": "MPLDUCH", "name": "DUCHEMIN", "type": "prescriber"}
                ],
                "request_id": "0012541258",
                "results": [
                    {
                        "type": "NM",
                        "code": "NA",
                        "date": datetime.datetime(2011, 1, 24, 13, 49),
                        "value": "12",
                    }
                ],
                "sample_id": "0100000667",
                "sex": "F",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_other(self):
    worklist = self.decode(
        """
    MSH|^~\\&|MPL||cobasInfinity||20191112101611||OML^O33^OML_O33|MPLORDER-65329-36970-000001|P|2.5|||AL|NE
    PID|1|0000077659|||TEST MPL^Connexion|ROCHE|20170207|M
    NTE|1|P||
    PV1|1||||||
    SPM|1|950000000003||ECBU
    ORC|NW|950000000003|950000000003
    OBR|1||950000000003|ECBU^ECBU||||||||||||^RDF|||||||||||^^^20191112101607^^R
    NTE|1|P|Infos tube|"""
    )
    expected = {
        "context": "order",
        "receiver_id": "cobasInfinity",
        "sender_id": "MPL",
        "samples": [
            {
                "birthdate": datetime.datetime(2017, 2, 7, 0, 0),
                "collection_date": datetime.datetime(2019, 11, 12, 10, 16, 7),
                "first_name": "Connexion",
                "last_name": "TEST MPL",
                "location": None,
                "maiden_name": "ROCHE",
                "orders": [{"action": "add", "code": "ECBU", "priority": "R"}],
                "patient_id": "0000077659",
                "request_id": "950000000003",
                "sample_id": "950000000003",
                "sex": "M",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_encode_result(self):
    result_worklist = copy.deepcopy(self.result_worklist)
    message = self.result(
        worklist=result_worklist, context={"control_id": "1521651964637"}
    )
    expected = """
    MSH|^~\\&|MCA||MPL||20130715141912||OUL^R22^OUL_R22|1521651964637|P|2.5|||NE|AL
    PID|1||||NYME^ANO|NJF|20110513|F
    PV1|1||LOC||||ATTENDING
    SPM|1|123456
    OBR|1||123456|A^A||||||||||||PRESCRIBER^PRESCRIBER|||||||||||^^^20130513114242
    OBX|1|NM|A||42|mV|||||F|||20110506121400||VAL||INST
    OBR|2||123456|B^B||||||||||||PRESCRIBER^PRESCRIBER|||||||||||^^^20130513114242
    OBX|1|CE|B||NORESULT|mV|||||F|||20110506121400||VAL||INST
    OBR|3||123456|C^C||||||||||||PRESCRIBER^PRESCRIBER|||||||||||^^^20130513114242
    OBX|1|NM|C||42|mV|||||F|||20110506121400||VAL||INST
    """
    assert message == expected
