# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "multistat"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||MCA|||||||P|LIS2A2|20130715141912
    P|1|036666666|||NYME^ANO||20110513|F
    O|1|123456||^^^A|S|20130603110342
    O|2|123456||^^^B|S|20130603110342
    L|1|N
    """
    assert message == expected


def test_encode_void_order(self):
    message = self.encode(
        self.void_worklist,
        context={
            "query": self.parse(
                """
            H|\\^&||EV29||||||||||20050411143805
            Q|1|A2||||||||||N
            L|1|N
            """
            )
        },
    )
    expected = """
        H|\\^&|||MCA|||||||P|LIS2A2|20130715141912
        P|1
        O|1|A2|||||||||||||||||||||||Y
        L|1|N
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&||EV29||||||||||20050411143805
    Q|1|A2||||||||||N
    L|1|N
    """
    )
    expected = {"context": "query", "samples": [{"sample_id": "A2", "type": "patient"}]}
    assert worklist == expected


def test_decode_results(self):
    date = datetime.datetime(2017, 7, 26, 10, 41, 31)
    worklist = self.decode(
        """
    H|\\^&|||052-16-001|||||||P|LIS2A2|20170727114239
    O|1|110133|123|^^^EMU_41218|S|20170727114239|||||N||||10047^Drugs of Abuse Urine||||||||||F|||||
    R|1|^^^EMU_41218|Negative||100ng/ml|||F||rdx.manufacture|20170726104131|20170726104131|123
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "EMU_41218",
                        "date": date,
                        "flags": [],
                        "instrument": "123",
                        "operator": "rdx.manufacture",
                        "value": "Negative",
                    }
                ],
                "sample_id": "110133",
                "specimen": "10047",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    date = datetime.datetime(2017, 7, 26, 10, 26, 47)
    worklist = self.decode(
        """
    H|\\^&|||052-16-001|||||||P|LIS2A2|20170727103121
    O|1|110091|123|^^^EMU_41225|S|20170727103121|||||Q||||10047^Drugs of Abuse Urine||||||||||F|||||
    R|1|^^^EMU_41225|Positive||10ng/ml|||F||rdx.manufacture|20170726102647|20170726102647|123
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "110091",
                "results": [
                    {
                        "code": "EMU_41225",
                        "date": date,
                        "flags": [],
                        "instrument": "123",
                        "operator": "rdx.manufacture",
                        "value": "Positive",
                    }
                ],
                "sample_id": "110091",
                "type": "control",
            }
        ],
    }
    assert worklist == expected
